﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Diagnostics;
using System.Web.UI.WebControls.WebParts;
using System.ComponentModel;
using System.Xml;

using Microsoft.Office.Server.Search.WebControls;
using Microsoft.Office.Server.Search.Administration;
using Microsoft.Office.Server.Search.Query;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;


namespace SortedResultsPart
{
    public class SearchResultsPart : CoreResultsWebPart, IWebEditable
    {

        //our sort drop down list; using a custom class to control naming of the control because
        //with the page lifecycle we don't get the value from the DD until after the query datasource is created
        private SearchDropList sortDD = new SearchDropList(mFormSortField);
        private SearchDropList sortDir = new SearchDropList(mFormSortDirection);
        
        //our array of sortable fields
        private string[] values = null;

        //track what was chosen so we can select the right item in the drop down list on postback
        private string mCurValue = string.Empty;
        private string mCurDir = string.Empty;

        //where we store the postback values for sort field and direction
        protected static string mFormSortField = "481D954C-F192-4762-9278-D926CF1A3FB7";
        protected static string mFormSortDirection = "A9AEF018-7C49-4f45-8AA8-6F5F241575BF";


        //the list of fields we want to be able to sort by
        [WebBrowsable(false), Category("Sort Options"), Personalizable(PersonalizationScope.Shared), 
        WebDescription("Select the fields available for sorting search results")]
        public string SortFields { get; set; }


        //whether we should always tack on our custom query criteria to the user's query
        [WebBrowsable(true), Category("Sorted Results Author Options"), Personalizable(PersonalizationScope.Shared),
        WebDescription("Should results always be restricted to only show items authored by Steve"), 
        FriendlyName("Only Show Items Authored By Steve")]
        public bool OnlyShowSteve { get; set; }


        public SearchResultsPart()
        {
            //default constructor
        }

        protected override void ConfigureDataSourceProperties()
        {
            base.ConfigureDataSourceProperties();
        }

        protected override void CreateChildControls()
        {
            try
            {
                base.CreateChildControls();

                //populate our sort drop down
                if (!string.IsNullOrEmpty(SortFields))
                    values = SortFields.Split(";".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

                if ((values != null) && (values.Count() > 0))
                {
                    //store the values that should be selected in the drop downs
                    int curIndex = 0;

                    if (Page.IsPostBack)
                    {
                        mCurValue = Page.Request.Form[SearchResultsPart.mFormSortField];
                        mCurDir = Page.Request.Form[SearchResultsPart.mFormSortDirection];
                    }
                    else
                    {
                        //if it isn't a post back then just take the first values
                        mCurValue = values[0];
                        mCurDir = "ASC";
                    }

                    //enumerate values and add them to the drop down
                    foreach (string value in values)
                    {
                        sortDD.Items.Add(value);
                    }

                    //create a div to hold the label, drop down of sort fields and sort direction
                    System.Web.UI.HtmlControls.HtmlGenericControl ctrlDiv = new HtmlGenericControl("DIV");

                    //set the text next to the drop down and enable auto post back so we change sort order
                    //when a new item is selected from the drop down
                    System.Web.UI.WebControls.Label ddLbl = new System.Web.UI.WebControls.Label();
                    ddLbl.Text = "Sort By:";
                    ctrlDiv.Controls.Add(ddLbl);

                    sortDD.SelectedIndex = curIndex;
                    ctrlDiv.Controls.Add(sortDD);

                    //add the sort directions and then the sort dropdown
                    sortDir.Items.Add(new System.Web.UI.WebControls.ListItem("Ascending", "ASC"));
                    sortDir.Items.Add(new System.Web.UI.WebControls.ListItem("Descending", "DESC"));
                    ctrlDiv.Controls.Add(sortDir);

                    //add the button to actually re-execute the query with the selected sort field and direction
                    System.Web.UI.WebControls.ImageButton goBtn = new System.Web.UI.WebControls.ImageButton();
                    goBtn.ImageUrl = "/_layouts/images/Forward.gif";
                    goBtn.ImageAlign = System.Web.UI.WebControls.ImageAlign.AbsMiddle;
                    ctrlDiv.Controls.Add(goBtn);

                    //add the div
                    this.Controls.AddAt(0, ctrlDiv);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error: " + ex.Message);
            }
        }


        protected override void Render(HtmlTextWriter writer)
        {
            //if we're posting back, then pick the user's last selection
            if (this.Page.IsPostBack)
            {
                sortDD.SelectedValue = mCurValue;
                sortDir.SelectedValue = mCurDir;
            }

            //*********************************************************
            //if you wanted to do some custom rendering of your own
            //you can get the xmlResponseDoc property and transform it
            //however you want in the Render method; just don't call 
            //the base.Render method when you are done
            //XmlDocument xDoc = this.xmlResponseDoc;
            //Debug.Write(xDoc.OuterXml);
            //*********************************************************

            base.Render(writer);
        }


        protected override void CreateDataSource()
        {
            this.DataSource = new MyCoreResultsDataSource(this);
        }


        public class SearchDropList : System.Web.UI.WebControls.DropDownList
        {

            //default constructor
            public SearchDropList(string ControlID)
            {
                //configure it to use our name so we can check on postback
                this.ID = ControlID;
                this.Attributes.Add("Name", ControlID);
                this.AutoPostBack = false;
            }

            //override naming constructor
            public override System.Web.UI.Control NamingContainer
            {
                get
                {
                    //return base.NamingContainer;
                    return null;
                }
            }
        }


        public class MyCoreResultsDataSource : CoreResultsDatasource    
        {
            public MyCoreResultsDataSource(CoreResultsWebPart ParentWebpart)
                            : base(ParentWebpart)
            {
                //create the View that will be used with this datasource
                this.View = new MyCoreResultsDataSourceView(this, "MyCoreResults");
            }
        }


        public class MyCoreResultsDataSourceView : CoreResultsDatasourceView    
        {

            public MyCoreResultsDataSourceView(SearchResultsBaseDatasource DataSourceOwner, string ViewName)
                            : base(DataSourceOwner, ViewName)
            {
                //make sure we have a value for the datasource
                if (DataSourceOwner == null)
                {
                    throw new ArgumentNullException("DataSourceOwner");
                }

                //get a typed reference to our datasource
                MyCoreResultsDataSource ds = this.DataSourceOwner as MyCoreResultsDataSource;

                //configure the query manager for this View
                this.QueryManager = SharedQueryManager.GetInstance(ds.ParentWebpart.Page).QueryManager;
            }


            public override void SetPropertiesOnQdra()
            {
                //overriding this just so I can see what the query looks like that ends up getting submitted
                base.SetPropertiesOnQdra();
                Debug.WriteLine(this.QueryManager.UserQuery);
            }


            //public override System.Xml.XPath.XPathNavigator GetXPathNavigator(DataSourceSelectArguments selectArguments)
            //{
            //    SearchResultsPart wp = this.DataSourceOwner.ParentWebpart as SearchResultsPart;
            //    //if (wp.OnlyShowSteve)
            //        this.QueryManager.UserQuery += " AND (Author:\"Steve Peschka\")";

            //    return base.GetXPathNavigator(selectArguments);           
            //}

            public override void AddSortOrder(SharePointSearchRuntime runtime)
            {
                #region Ensure Runtime
                //make sure our runtime has been properly instantiated
                if (runtime.KeywordQueryObject == null)
                {
                    return;
                }
                #endregion

                //remove any other sorted fields we might have had
                runtime.KeywordQueryObject.SortList.Clear();

                //get the datasource so we can get to the web part and retrieve the sort fields the user selected
                SearchResultsPart wp = this.DataSourceOwner.ParentWebpart as SearchResultsPart;
                string sortField = wp.SortFields;

                //check to see if any sort fields have been provided
                if (!string.IsNullOrEmpty(sortField))
                {
                    //if posting back then use the value from the sort drop down
                    if (wp.Page.IsPostBack)
                    {
                        //get the sort direction that was selected in the drop down
                        SortDirection dir =
                            (wp.Page.Request.Form[SearchResultsPart.mFormSortDirection] == "ASC" ? 
                            SortDirection.Ascending : SortDirection.Descending);

                        //configure the sort list with the sort field and direction
                        runtime.KeywordQueryObject.SortList.Add(wp.Page.Request.Form[SearchResultsPart.mFormSortField], 
                            dir);
                    }
                    else
                    {
                        //split the value out from it's delimiter and take the first item in descending order
                        string[] values = sortField.Split(";".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                        runtime.KeywordQueryObject.SortList.Add(values[0], SortDirection.Descending);
                    }
                }
                else  //no sort fields provided so use the default sort order
                    base.AddSortOrder(runtime);

                //**********************************************
                //add custom criteria for my special web part
                //**********************************************
                if (wp.OnlyShowSteve)
                    this.DataSourceOwner.Query += " AND (Author:\"Steve Peschka\")";

                #region DataSource Properties
                try
                {
                    MyCoreResultsDataSource ds = this.DataSourceOwner as MyCoreResultsDataSource;
                    Debug.WriteLine(ds.EnableNicknames);
                    Debug.WriteLine(ds.EnablePhonetic);
                    Debug.WriteLine(ds.EnableStemming);
                    Debug.WriteLine(ds.IgnoreNoise);
                    Debug.WriteLine(ds.Location);
                    Debug.WriteLine(ds.PropertiesToRetrieve);
                    Debug.WriteLine(ds.Query);
                    Debug.WriteLine(ds.QueryLanguage.EnglishName);
                    Debug.WriteLine(ds.RemoveDuplicates);
                    Debug.WriteLine(ds.ResultsPerPage);
                    Debug.WriteLine(ds.StartAt);
                    Debug.WriteLine(ds.SupplementalQuery);
                    Debug.WriteLine(ds.UseLocationVisualization);
                }
                catch (Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
                #endregion

            }
        }


        #region IWebEditable Members

        EditorPartCollection IWebEditable.CreateEditorParts()
        {
            List<EditorPart> editorParts = new List<EditorPart>();
            SearchResultsPropertyPicker srp = new SearchResultsPropertyPicker();
            srp.ID = this.ID + "SearchResultsPropertyPicker";
            editorParts.Add(srp);
            return new EditorPartCollection(editorParts);
        }

        object IWebEditable.WebBrowsableObject
        {
            get { return this; }
        }

        #endregion
    }


}

