﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Diagnostics;
using Microsoft.Office.Server.Search.WebControls;
using Microsoft.Office.Server.Search.Administration;
using Microsoft.Office.Server.Search.Query;
using Microsoft.Office.Server.Search;
using Microsoft.SharePoint.Administration;
using System.Web.UI.HtmlControls;


namespace SortedResultsPart
{
    class SearchResultsPropertyPicker : EditorPart, System.Web.UI.INamingContainer
    {

        private Panel fldList = null;

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            //set the title for the property group in the web part property editor
            this.Title = "Sortable Properties";

            //add the label and picker control here, and fill it in SyncChanges
            Label lbl = new Label();
            lbl.Text = "Select from the sortable fields in this search application:";
            this.Controls.Add(lbl);

            fldList = new Panel();
            this.Controls.Add(fldList);
        }

        public override bool ApplyChanges()
        {
            try
            {
                EnsureChildControls();

                //get the part we're going to be working with
                SearchResultsPart sp = WebPartToEdit as SearchResultsPart;

                #region reviewing postback fields
                //get the values from post back
                //for (int i = 0; i < Page.Request.Form.Count; i++)
                //{
                //    Debug.WriteLine(Page.Request.Form.Keys[i] + " = " + Page.Request.Form[i]);
                //}
                #endregion

                var values = from string value in Page.Request.Form
                             where value.StartsWith("*")
                             select new
                             {
                                 PropKey = value
                             };

                //enumerate the results so we can store it in the web part property
                StringBuilder tempVal = new StringBuilder(1024);
                foreach (var value in values)
                {
                    //Debug.WriteLine(value.PropKey);
                    //replace the form name delimiter with the web part field delimiter
                    tempVal.Append(value.PropKey.Replace("*", ";"));
                }
                
                //plug into the web part property bag
                sp.SortFields = tempVal.ToString();

                return true;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                return false;
            }
        }


        public override void SyncChanges()
        {
            try
            {
                EnsureChildControls();

                //get the properties from the web part and plug them into our UI
                SearchResultsPart sp = WebPartToEdit as SearchResultsPart;

                //get the sort fields from the web part
                string sortVals = sp.SortFields;

                //array used to hold the fields that have been selected
                string[] values = null;

                //if a value has been stored in the property, then split it into an array
                if (!string.IsNullOrEmpty(sortVals))
                    values = sortVals.Split(";".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

                //get the list of sortable fields
                //run with privileges since we need to get to search admin objects
                SPSecurity.RunWithElevatedPrivileges(delegate()
                {
                    //start by getting the search application
                    #region wip
                    //SPServiceContext svcCtx = SPServiceContext.Current;
                    //SearchServiceApplicationProxy pxy = (SearchServiceApplicationProxy)SearchServiceApplicationProxy.GetProxy(svcCtx);
                    #endregion
                    SearchServiceApplication ssa = SearchContext.Current.SearchApplication;

                    //get the schema
                    Schema s = new Schema(ssa);

                    //select the sortable properties from the schema
                    var mp = from ManagedProperty mProp in s.AllManagedProperties
                             where (
                             (
                             (mProp.ManagedType == ManagedDataType.Text) &&
                             (mProp.HasMultipleValues == false) &&
                             (mProp.IsInDocProps == true) &&
                             (mProp.MaxCharactersInPropertyStoreIndex > 0)
                             ) |
                             (
                             (mProp.ManagedType == ManagedDataType.DateTime) |
                             (mProp.ManagedType == ManagedDataType.Integer)
                             )
                             )
                             orderby mProp.Name
                             select new
                             {
                                 DisplayName = mProp.Name,
                                 EnabledForScoping = mProp.EnabledForScoping,
                                 ManagedType = mProp.ManagedType.ToString(),
                                 Pid = mProp.PID
                             };

                    //enumerate and populate checkbox list
                    foreach (var prop in mp)
                    {
                        //create a new list item with the property name
                        SearchListItem li = new SearchListItem(prop.DisplayName, prop.DisplayName);

                        //if we have values in the property then checked to see if this is one that was previously selected
                        if (!string.IsNullOrEmpty(sortVals))
                        {
                            if (values.Contains(prop.DisplayName))
                                li.Checked = true;
                        }

                        //add the item to our picker
                        fldList.Controls.Add(li);
                    }
                });
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }
    }


    public class SearchListItemCheck : HtmlInputCheckBox
    {
        //default constructor
        public SearchListItemCheck()
        {
        }
        
        public override System.Web.UI.Control NamingContainer
        {
            get
            {
                //return base.NamingContainer;
                return null;
            }
        }
    }

    public class SearchListItem : HtmlGenericControl
    {

        private Label chkLbl = new Label();
        private SearchListItemCheck chkBox = new SearchListItemCheck();

        //default constructor
        public SearchListItem()
        { 
        }

        public SearchListItem(string Text, string Value)
        {
            //set the tag type for our control
            this.TagName = "div";

            //plug in the values and ID
            chkLbl.Text = Text;

            chkBox.Value = Value;
            chkBox.ID = "*" + Value;
            chkBox.Name = "*" + Value;

            //add the controls to our container
            this.Controls.Add(chkBox);
            this.Controls.Add(chkLbl);
        }

        public bool Checked
        {
            get
            {
                return this.chkBox.Checked;
            }
            set
            {
                this.chkBox.Checked = value;
            }
        }



    }
}
